package com.qmth.eds.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.Result;
import com.qmth.eds.common.util.ResultUtil;
import com.qmth.eds.job.service.JobService;
import com.qmth.eds.service.CloudMarkingScoreService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

/**
 * <p>
 * 云阅卷成绩 前端控制器
 * </p>
 */
@Api(tags = "成绩接口")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/cloud_marking_score")
public class CloudMarkingScoreController {

    @Resource
    private CloudMarkingScoreService cloudMarkingScoreService;

    @Resource
    private JobService jobService;

    /**
     * 同步云阅卷考试
     */
    @ApiOperation(value = "同步考生成绩")
    @PostMapping("/sync_student_score")
    public Result syncStudentScore(Long semesterId, Long examTypeId, Long collegeId, Integer examId) {
        if (semesterId == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择学期");
        }
        if (examTypeId == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择考试类型");
        }
        if (examId == null) {
            throw ExceptionResultEnum.ERROR.exception("请选择云阅卷考试");
        }
        jobService.syncStudentScore(semesterId, examTypeId, collegeId, examId);
        return ResultUtil.success(true);
    }

    /**
     * 查询
     *
     * @param semesterId 学期ID
     * @param examTypeId 考试类型ID
     * @param examId     云阅卷考试ID
     * @param pageNumber 分页参数
     * @param pageSize   分页参数
     */
    @ApiOperation(value = "查询")
    @PostMapping("/page")
    public Result page(@RequestParam(value = "semesterId", required = false) Long semesterId,
                       @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                       @RequestParam(value = "collegeId", required = false) Long collegeId,
                       @RequestParam(value = "examId", required = false) String examId,
                       @RequestParam(value = "syncCourseCode", required = false) String syncCourseCode,
                       @RequestParam(value = "cloudMarkingCourseCode", required = false) String cloudMarkingCourseCode,
                       @RequestParam(value = "studentCode", required = false) String studentCode,
                       @RequestParam(value = "related", required = false) Boolean related,
                       @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(cloudMarkingScoreService.pageData(collegeId, semesterId, examTypeId, examId, syncCourseCode, cloudMarkingCourseCode, studentCode, related, pageNumber, pageSize));
    }
}
