package com.qmth.eds.api;


import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.eds.bean.result.AssignContrastResult;
import com.qmth.eds.bean.result.AssignResultPreviewResult;
import com.qmth.eds.bean.result.EditResult;
import com.qmth.eds.bean.result.StudentMarkResult;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.ExamAssign;
import com.qmth.eds.common.util.Result;
import com.qmth.eds.common.util.ResultUtil;
import com.qmth.eds.service.CloudMarkingScoreService;
import com.qmth.eds.service.ExamAssignService;
import io.swagger.annotations.*;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 赋分管理 前端控制器
 * </p>
 */
@Api(tags = "赋分接口")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/assign")
@Validated
public class ExamAssignController {

    @Resource
    private ExamAssignService examAssignService;

    @Resource
    CloudMarkingScoreService cloudMarkingScoreService;

    @ApiOperation(value = "分页查询")
    @PostMapping("/page")
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result pageData(@RequestParam(value = "semesterId", required = false) Long semesterId,
                           @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                           @RequestParam(value = "collegeId", required = false) Long collegeId,
                           @RequestParam(value = "courseCode", required = false) String courseCode,
                           @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                           @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(examAssignService.pageData(semesterId, examTypeId, collegeId, courseCode, pageNumber, pageSize));
    }

    @ApiOperation(value = "准备试算")
    @PostMapping("/to_calc")
    @ApiResponses({@ApiResponse(code = 200, message = "查询成功", response = Result.class)})
    public Result toCalc(@RequestParam(value = "semesterId", required = false) Long semesterId,
                         @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                         @RequestParam(value = "collegeId", required = false) Long collegeId,
                         @RequestParam(value = "syncCourseCode", required = false) String syncCourseCode,
                         @RequestParam(value = "courseCode", required = false) String courseCode,
                         @RequestParam(value = "courseName", required = false) String courseName,
                         @RequestParam(value = "openCollege", required = false) String openCollege) {
        return ResultUtil.ok(examAssignService.toCalc(semesterId, examTypeId, collegeId, syncCourseCode, courseCode, courseName, openCollege));
    }


    @ApiOperation(value = "计算")
    @PostMapping("/calc")
    @ApiResponses({@ApiResponse(code = 200, message = "保存/更新成功", response = EditResult.class)})
    public Result save(@RequestBody String data) {
        return ResultUtil.ok(examAssignService.calcData(data), "");
    }

    @ApiOperation(value = "发布赋分成绩")
    @PostMapping("/publish")
    @ApiResponses({@ApiResponse(code = 200, message = "保存/更新成功", response = EditResult.class)})
    public Result publish(@RequestParam(value = "id", required = false) Long id,
                          @RequestParam(value = "semesterId", required = false) Long semesterId,
                          @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                          @RequestParam(value = "collegeId", required = false) Long collegeId,
                          @RequestParam(value = "syncCourseCode", required = false) String syncCourseCode,
                          @RequestParam(value = "courseCode", required = false) String courseCode,
                          @RequestParam(value = "courseName", required = false) String courseName,
                          @RequestParam(value = "openCollege", required = false) String openCollege) {
        return ResultUtil.ok(examAssignService.publish(id, semesterId, examTypeId, collegeId, syncCourseCode, courseCode, courseName, openCollege));
    }

    @ApiOperation(value = "计算结果查询")
    @PostMapping("/result")
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = AssignResultPreviewResult.class)})
    public Result assignResultPreview(@ApiParam(value = "学期id", required = true) @RequestParam Long semesterId,
                                      @ApiParam(value = "考试id", required = true) @RequestParam Long examTypeId,
                                      @ApiParam(value = "学院id", required = true) @RequestParam Long collegeId,
                                      @ApiParam(value = "科目代码", required = true) @RequestParam String courseCode,
                                      @ApiParam(value = "开课学院", required = true) @RequestParam String openCollege,
                                      @ApiParam(value = "考查学院", required = false) @RequestParam(required = false) String kcCollege,
                                      @ApiParam(value = "班级id", required = false) @RequestParam(required = false) String clazzId,
                                      @ApiParam(value = "任课老师工号", required = false) @RequestParam(required = false) String teacherCode,
                                      @ApiParam(value = "赋分分数段（开始)", required = false) @RequestParam(required = false) String minAssignScore,
                                      @ApiParam(value = "赋分分数段（结束)", required = false) @RequestParam(required = false) String maxAssignScore,
                                      @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                                      @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        IPage<AssignResultPreviewResult> assignResultPreviewResultIPage = examAssignService.assignResultPreview(new Page<>(pageNumber, pageSize), semesterId, examTypeId, collegeId, courseCode, openCollege, kcCollege, clazzId, teacherCode, minAssignScore, maxAssignScore);
        if (Objects.nonNull(assignResultPreviewResultIPage) && !CollectionUtils.isEmpty(assignResultPreviewResultIPage.getRecords())) {
            for (AssignResultPreviewResult a : assignResultPreviewResultIPage.getRecords()) {
                a.setScoreDetail("客观题评分明细：" + a.getObjectiveScoreDetail() + "\r\n" + "主观题评分明细：" + a.getSubjectiveScoreDetail());
            }
        }
        return ResultUtil.ok(assignResultPreviewResultIPage);
    }

    @ApiOperation(value = "计算结果对比")
    @PostMapping("/contrast")
    @ApiResponses({@ApiResponse(code = 200, message = "保存/更新成功", response = AssignContrastResult.class)})
    public Result contrast(@ApiParam(value = "学期id", required = true) @RequestParam Long semesterId,
                           @ApiParam(value = "考试id", required = true) @RequestParam Long examTypeId,
                           @ApiParam(value = "学院id", required = true) @RequestParam Long collegeId,
                           @ApiParam(value = "科目代码", required = true) @RequestParam String courseCode,
                           @ApiParam(value = "开课学院", required = true) @RequestParam String openCollege,
                           @ApiParam(value = "及格分", required = true) @RequestParam BigDecimal passScore) {
        List<StudentMarkResult> studentMarkResultList = cloudMarkingScoreService.queryStudentMark(semesterId, examTypeId, collegeId, openCollege, courseCode);
        AssignContrastResult assignContrastResult = null;
        if (!CollectionUtils.isEmpty(studentMarkResultList)) {
            BigDecimal assignBeforeSumScore = new BigDecimal(0), assignAfterSumScore = new BigDecimal(0), sumCount = new BigDecimal(studentMarkResultList.size());
            Integer assignBeforeFailCount = 0, assignAfterFailCount = 0;
            for (StudentMarkResult s : studentMarkResultList) {
                //赋分前分数和不及格人数
                if (Objects.nonNull(s.getTotalScore()) && !Objects.equals(s.getTotalScore().trim(), "")) {
                    BigDecimal totalScore = new BigDecimal(s.getTotalScore());
                    assignBeforeSumScore = assignBeforeSumScore.add(totalScore);
                    if (totalScore.compareTo(passScore) < 0) {
                        assignBeforeFailCount++;
                    }
                }
                //赋分后分数和不及格人数
                if (Objects.nonNull(s.getAssignScore()) && !Objects.equals(s.getAssignScore().trim(), "")) {
                    BigDecimal assignScore = new BigDecimal(s.getAssignScore());
                    assignAfterSumScore = assignAfterSumScore.add(assignScore);
                    if (assignScore.compareTo(passScore) < 0) {
                        assignAfterFailCount++;
                    }
                }
            }
            //赋分前平均分
            BigDecimal assignBeforeAverageScore = assignBeforeSumScore.divide(sumCount, 2, RoundingMode.HALF_UP).setScale(2, RoundingMode.HALF_UP);
            //赋分前不及格率
            BigDecimal assignBeforeFailRate = new BigDecimal(assignBeforeFailCount).divide(sumCount, 2, RoundingMode.HALF_UP).setScale(2, RoundingMode.HALF_UP);
            //赋分后平均分
            BigDecimal assignAfterAverageScore = assignAfterSumScore.divide(sumCount, 2, RoundingMode.HALF_UP).setScale(2, RoundingMode.HALF_UP);
            //赋分后不及格率
            BigDecimal assignAfterFailRate = new BigDecimal(assignAfterFailCount).divide(sumCount, 2, RoundingMode.HALF_UP).setScale(2, RoundingMode.HALF_UP);
            assignContrastResult = new AssignContrastResult(sumCount.intValue(), assignBeforeAverageScore, assignBeforeFailRate,
                    assignBeforeFailCount, assignAfterAverageScore, assignAfterFailRate, assignAfterFailCount);
        }
        return ResultUtil.ok(Objects.isNull(assignContrastResult) ? new AssignContrastResult() : assignContrastResult);
    }

    /**
     * 导出
     *
     * @param semesterId  学期
     * @param examTypeId  考试类型
     * @param collegeId   学院ID
     * @param courseCode  课程代码
     * @param openCollege 开课学院
     * @param response    response
     */
    @ApiOperation(value = "导出赋分成绩")
    @PostMapping("/download")
    public void page(@RequestParam(value = "id", required = false) Long id,
                     @RequestParam(value = "semesterId", required = false) Long semesterId,
                     @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                     @RequestParam(value = "collegeId", required = false) Long collegeId,
                     @RequestParam(value = "courseCode", required = false) String courseCode,
                     @RequestParam(value = "openCollege", required = false) String openCollege,
                     HttpServletResponse response) {
        examAssignService.download(id, semesterId, examTypeId, collegeId, courseCode, openCollege, response);
    }
}
