package com.qmth.eds.api;

import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.enums.UseSceneEnum;
import com.qmth.eds.service.ExamDownloadRecordService;
import com.qmth.eds.common.util.Result;
import com.qmth.eds.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;

@Api(tags = "下载记录接口")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/exam_download_record")
public class ExamDownloadRecordController {

    @Resource
    private ExamDownloadRecordService examDownloadRecordService;

    /**
     * 查询
     *
     * @param operateTime 下载时间
     * @param used        是否使用
     * @param useScene    下载用途
     * @param pageNumber  分页参数
     * @param pageSize    分页参数
     */
    @ApiOperation(value = "查询")
    @PostMapping("/page")
    public Result page(@RequestParam(value = "semesterId", required = false) Long semesterId,
                       @RequestParam(value = "examTypeId", required = false) Long examTypeId,
                       @RequestParam(value = "operateTime", required = false) Long operateTime,
                       @RequestParam(value = "used", required = false) Boolean used,
                       @RequestParam(value = "useScene", required = false) String useScene,
                       @RequestParam @Min(SystemConstant.PAGE_NUMBER_MIN) Integer pageNumber,
                       @RequestParam @Min(SystemConstant.PAGE_SIZE_MIN) @Max(SystemConstant.PAGE_SIZE_MAX) Integer pageSize) {
        return ResultUtil.ok(examDownloadRecordService.list(semesterId, examTypeId, operateTime, used, useScene, pageNumber, pageSize));
    }

    /**
     * 用途列表
     */
    @ApiOperation(value = "用途")
    @PostMapping("/list_use_scene")
    public Result listUseScene() {
        return ResultUtil.ok(UseSceneEnum.listTypes());
    }

    /**
     * 标记用途
     *
     * @param id       id
     * @param used     是否使用
     * @param useScene 下载用途
     */
    @ApiOperation(value = "标记用途")
    @PostMapping("/used")
    public Result used(@RequestParam(value = "id") String id,
                       @RequestParam(value = "used") Boolean used,
                       @RequestParam(value = "useScene") String useScene) {
        return ResultUtil.ok(examDownloadRecordService.used(id, used, useScene));
    }

}
