package com.qmth.eds.api;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.eds.bean.params.LoginParam;
import com.qmth.eds.bean.result.*;
import com.qmth.eds.common.entity.CloudMarkingScore;
import com.qmth.eds.common.entity.ExamSyncStudent;
import com.qmth.eds.common.entity.SysUser;
import com.qmth.eds.common.enums.AppSourceEnum;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.EhcacheUtil;
import com.qmth.eds.common.util.Result;
import com.qmth.eds.common.util.ResultUtil;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.service.*;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.security.NoSuchAlgorithmException;
import java.time.temporal.ChronoUnit;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 系统公共Controller
 */
@Api(tags = "系统接口")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/common")
public class SysController {

    @Resource
    SysUserService sysUserService;

    @Resource
    TeachcloudCommonService teachcloudCommonService;

    @Resource
    ExamSyncStudentService examSyncStudentService;

    @Resource
    CommonService commonService;

    @Resource
    CommonCacheService commonCacheService;

    @Resource
    EhcacheUtil ehcacheUtil;

    /**
     * 登录
     *
     * @param login         登录参数
     * @param bindingResult 错误
     */
    @ApiOperation(value = "登录")
    @PostMapping("/login")
    @ApiResponses({@ApiResponse(code = 200, message = "用户信息", response = LoginResult.class)})
    @Aac(auth = false)
    public Result login(@ApiParam(value = "用户信息", required = true) @Valid @RequestBody LoginParam login, BindingResult bindingResult) throws NoSuchAlgorithmException {
        if (bindingResult.hasErrors()) {
            return ResultUtil.error(bindingResult.getAllErrors().get(0).getDefaultMessage());
        }

        // 账号密码登录
        String loginName = login.getLoginName();
        String password = login.getPassword();
        // 非空校验
        if (StringUtils.isBlank(loginName)) {
            throw ExceptionResultEnum.ERROR.exception("用户名不能为空");
        }
        if (StringUtils.isBlank(password)) {
            throw ExceptionResultEnum.ERROR.exception("密码不能为空");
        }
        QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
        wrapper.lambda().and(w -> w.eq(SysUser::getLoginName, loginName).or().eq(SysUser::getCode, loginName));
        List<SysUser> userList = sysUserService.list(wrapper);
        //用户不存在
        if (userList == null || userList.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("用户不存在");
        }
        if (userList.size() > 1) {
            throw ExceptionResultEnum.ERROR.exception("查出多个用户");
        }
        SysUser sysUser = userList.get(0);
        if (!sysUser.getEnable()) {
            throw ExceptionResultEnum.ERROR.exception("用户被禁用");
        }
        if (!password.equals(userList.get(0).getPassword())) {
            throw ExceptionResultEnum.ERROR.exception("密码错误");
        }
        LoginResult loginResult = teachcloudCommonService.login(login.getPassword(), sysUser, AppSourceEnum.SYSTEM);
        return ResultUtil.ok(loginResult);
    }

    /**
     * 登出
     */
    @ApiOperation(value = "登出")
    @PostMapping("/logout")
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = EditResult.class)})
    public Result logout() throws NoSuchAlgorithmException {
        Long userId = ServletUtil.getRequestUserId();
        teachcloudCommonService.removeUserInfo(userId, false);
        return ResultUtil.ok();
    }

    @ApiOperation(value = "查询科目列表")
    @PostMapping("/course/list")
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = ExamineCollegeResult.class)})
    public Result examineCollegeList(@ApiParam(value = "学期id") @RequestParam(required = false) Long semesterId,
                                     @ApiParam(value = "考试类型id") @RequestParam(required = false) Long examTypeId,
                                     @ApiParam(value = "学院ID") @RequestParam(required = false) Long collegeId) {
        List<CourseResult> examSyncStudentList = commonService.conditionCourseQuery(semesterId, examTypeId, collegeId);
        return ResultUtil.ok(!CollectionUtils.isEmpty(examSyncStudentList) ? examSyncStudentList : Collections.emptyList());
    }

    @ApiOperation(value = "查询考查学院列表")
    @PostMapping("/examine/college/list")
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = ExamineCollegeResult.class)})
    public Result examineCollegeList(@ApiParam(value = "学期id", required = true) @RequestParam Long semesterId,
                                     @ApiParam(value = "考试id", required = true) @RequestParam Long examTypeId,
                                     @ApiParam(value = "科目代码", required = true) @RequestParam String courseCode,
                                     @ApiParam(value = "开课学院", required = true) @RequestParam String openCollege,
                                     @ApiParam(value = "开课学院ID", required = true) @RequestParam Long collegeId) {
        List<CloudMarkingScore> examSyncStudentList = examSyncStudentService.queryByExamineCollegeList(semesterId, examTypeId, courseCode, openCollege, collegeId,null, null);
        List<ExamineCollegeResult> examineCollegeResultList = new ArrayList<>();
        if (!CollectionUtils.isEmpty(examSyncStudentList)) {
            for (CloudMarkingScore e : examSyncStudentList) {
                if (StringUtils.isNotBlank(e.getCollege())) {
                    examineCollegeResultList.add(new ExamineCollegeResult(e.getCollege(), e.getCollege()));
                }
            }
        }
        examineCollegeResultList= examSyncStudentList.stream().filter(m->StringUtils.isNotBlank(m.getCollege())).map(m->new ExamineCollegeResult(m.getCollege(), m.getCollege())).distinct().collect(Collectors.toList());
        return ResultUtil.ok(!CollectionUtils.isEmpty(examineCollegeResultList) ? examineCollegeResultList : Collections.emptyList());
    }

    @ApiOperation(value = "查询班级列表")
    @PostMapping("/clazz/list")
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = ClazzResult.class)})
    public Result clazzList(@ApiParam(value = "学期id", required = true) @RequestParam Long semesterId,
                            @ApiParam(value = "考试id", required = true) @RequestParam Long examTypeId,
                            @ApiParam(value = "科目代码", required = true) @RequestParam String courseCode,
                            @ApiParam(value = "开课学院", required = true) @RequestParam String openCollege,
                            @ApiParam(value = "考查学院名称", required = true) @RequestParam String kcCollege) {
        List<CloudMarkingScore> examSyncStudentList = examSyncStudentService.queryByExamineCollegeList(semesterId, examTypeId, courseCode, openCollege, null, kcCollege, null);
        Set<ClazzResult> clazzResultList = null;
        if (!CollectionUtils.isEmpty(examSyncStudentList)) {
            clazzResultList = new LinkedHashSet<>(examSyncStudentList.size());
            for (CloudMarkingScore e : examSyncStudentList) {
                if (StringUtils.isNotBlank(e.getClassName())) {
                    clazzResultList.add(new ClazzResult(e.getClassName(), e.getClassName()));
                }
            }
        }
        return ResultUtil.ok(!CollectionUtils.isEmpty(clazzResultList) ? clazzResultList : Collections.emptyList());
    }

    @ApiOperation(value = "查询任课老师列表")
    @PostMapping("/teacher/list")
    @ApiResponses({@ApiResponse(code = 200, message = "返回信息", response = TeacherResult.class)})
    public Result teacherList(@ApiParam(value = "学期id", required = true) @RequestParam Long semesterId,
                              @ApiParam(value = "考试id", required = true) @RequestParam Long examTypeId,
                              @ApiParam(value = "科目代码", required = true) @RequestParam String courseCode,
                              @ApiParam(value = "开课学院", required = true) @RequestParam String openCollege,
                              @ApiParam(value = "考查学院名称", required = true) @RequestParam String kcCollege,
                              @ApiParam(value = "班级id", required = true) @RequestParam String clazzId) {
        List<CloudMarkingScore> examSyncStudentList = examSyncStudentService.queryByExamineCollegeList(semesterId, examTypeId, courseCode, openCollege, null, kcCollege, clazzId);
        Set<TeacherResult> teacherResultList = null;
        if (!CollectionUtils.isEmpty(examSyncStudentList)) {
            teacherResultList = new LinkedHashSet<>(examSyncStudentList.size());
            for (CloudMarkingScore e : examSyncStudentList) {
                if (StringUtils.isNotBlank(e.getTeacher())) {
                    teacherResultList.add(new TeacherResult(e.getTeacher(), e.getTeacher()));
                }
            }
        }
        return ResultUtil.ok(!CollectionUtils.isEmpty(teacherResultList) ? teacherResultList : Collections.emptyList());
    }

    @Aac(auth = false)
    @GetMapping("/test")
    public void test() {
        // 1
        String key = "testkey";
        String value = "测试ehcache";
        String set = commonCacheService.set(key, value);
        System.out.println("set:" + set);
        String get = commonCacheService.get(key);
        System.out.println("get:" + get);
        commonCacheService.remove(key);
        String get1 = commonCacheService.get(key);
        System.out.println("get1:" + get1);

        // 2
        String key1 = "key2";
        String value1 = "自定义cache";
        ehcacheUtil.set("ddd", key1, value1, 10, ChronoUnit.SECONDS);
        for (int i = 0; i < 20; i++) {
            System.out.println("value2:" + ehcacheUtil.get("ddd", key1));
            try {
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
    }
}
