package com.qmth.eds.service;

import com.qmth.eds.common.entity.ExamAssign;
import com.qmth.eds.common.entity.ExamScheduleTask;
import com.qmth.eds.common.entity.ExamSyncTotal;
import com.qmth.eds.common.entity.TBSyncTask;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.enums.TaskTypeEnum;
import com.qmth.eds.common.util.ServletUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

/**
 * 异步同步云阅卷
 */
@Service
public class AsyncTaskService {

    @Resource
    private WhuDataSyncService whuDataSyncService;

    @Resource
    private TBSyncTaskService tbSyncTaskService;

    @Resource
    private ExamSyncTotalService examSyncTotalService;

    /**
     * 同步考务数据
     */
    public void syncExamData(ExamScheduleTask examScheduleTask, boolean isAuto) {
        // 记录日志
        TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(examScheduleTask, TaskTypeEnum.WHU_TEACH_DATA, isAuto);
        whuDataSyncService.syncWuhanUniversityExamData(examScheduleTask, tbSyncTask, isAuto);
    }

    public void syncStudentScore(Long semesterId, Long examTypeId, Long collegeId, Integer examId) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        ExamSyncTotal examSyncTotal = examSyncTotalService.getBySemesterIdAndExamTypeIdAndUseFile(schoolId, semesterId, examTypeId, true);
        if (examSyncTotal == null) {
            throw ExceptionResultEnum.ERROR.exception("未设置使用数据文件，请联系管理员处理");
        }
        // 记录日志
        String remark = "云阅卷考试：" + examId;
        TBSyncTask tbSyncTask = tbSyncTaskService.saveTask(collegeId, semesterId, examTypeId, TaskTypeEnum.CLOUD_MARKING_STUDENT_SCORE, remark);
        whuDataSyncService.syncCloudMarkingStudentScoreData(schoolId, tbSyncTask, examId);
    }

    public void calcAssignScore(ExamAssign examAssign) {
        whuDataSyncService.calcAssignScore(examAssign);
    }
}
