package com.qmth.eds.service;

import com.qmth.eds.bean.auth.AuthBean;
import com.qmth.eds.bean.dto.AuthOrgInfoDto;
import com.qmth.eds.bean.dto.syssetting.SimpleObject;
import com.qmth.eds.bean.result.MenuResult;
import com.qmth.eds.common.entity.*;
import com.qmth.eds.common.enums.PrivilegePropertyEnum;

import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * @Description: 缓存操作service
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/25
 */
public interface CommonCacheService {

    /**
     * 添加角色缓存
     *
     * @param roleId
     * @return
     */
    public SysRole roleCache(Long roleId);

    /**
     * 修改角色缓存
     *
     * @param roleId
     * @return
     */
    public SysRole updateRoleCache(Long roleId);

    /**
     * 删除角色缓存
     *
     * @param roleId
     */
    public void removeRoleCache(Long roleId);

    /**
     * 添加用户菜单缓存
     *
     * @param userId
     * @return
     */
    public MenuResult userMenuCache(Long userId);

    /**
     * 修改用户菜单缓存
     *
     * @param userId
     * @return
     */
    public MenuResult updateUserMenuCache(Long userId);

    /**
     * 删除用户菜单缓存
     *
     * @param userId
     */
    public void removeUserMenuCache(Long userId);

    /**
     * 添加用户缓存
     *
     * @param userId
     * @return
     */
    public SysUser userCache(Long userId);

    /**
     * 修改用户缓存
     *
     * @param userId
     * @return
     */
    public SysUser updateUserCache(Long userId);

    /**
     * 删除用户缓存
     *
     * @param userId
     */
    public void removeUserCache(Long userId);

    /**
     * 添加用户鉴权缓存
     *
     * @param userId
     * @return
     */
    public AuthBean userAuthCache(Long userId);

    /**
     * 修改用户鉴权缓存
     *
     * @param userId
     * @return
     */
    public AuthBean updateUserAuthCache(Long userId);

    /**
     * 删除用户鉴权缓存
     *
     * @param userId
     */
    public void removeUserAuthCache(Long userId);

    /**
     * 添加学校缓存
     *
     * @param schoolId
     * @return
     */
    public BasicSchool schoolCache(Long schoolId);

    /**
     * 添加学校缓存
     *
     * @param code
     * @return
     */
    public BasicSchool schoolCache(String code);

    /**
     * 添加机构缓存
     *
     * @param orgId
     * @return
     */
    public SysOrg orgCache(Long orgId);

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    public Set<String> privilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @param schoolId
     * @return
     */
    public Set<String> privilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum, Long schoolId);

    /**
     * 修改鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    public Set<String> updatePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);


    /**
     * 删除鉴权url
     *
     * @param privilegePropertyEnum
     */
    public void removePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum);

    /**
     * 添加角色权限缓存
     *
     * @param roleId
     * @return
     */
    public List<SysRolePrivilege> rolePrivilegeCache(Long roleId);

    /**
     * 修改角色权限缓存
     *
     * @param roleId
     * @return
     */
    public List<SysRolePrivilege> updateRolePrivilegeCache(Long roleId);

    /**
     * 删除角色权限缓存
     *
     * @param roleId
     */
    public void removeRolePrivilegeCache(Long roleId);

    /**
     * 添加用户角色权限缓存
     *
     * @param userId
     * @return
     */
    public List<SysUserRole> userRolePrivilegeCache(Long userId);

    /**
     * 修改用户角色权限缓存
     *
     * @param userId
     * @return
     */
    public List<SysUserRole> updateUserRolePrivilegeCache(Long userId);

    /**
     * 删除用户角色权限缓存
     *
     * @param userId
     */
    public void removeUserRolePrivilegeCache(Long userId);

    AuthOrgInfoDto authInfoCache(String code);

    /**
     * 修改鉴权缓存
     *
     * @param code
     * @return
     */
    public AuthOrgInfoDto updateAuthInfoCache(String code);

    void removeAuthInfoCache(String code);

    /**
     * 删除学校缓存
     */
    public void removeSchoolIdCache();

    /**
     * 删除学校缓存
     */
    public void removeSchoolCodeCache();

    /**
     * 删除机构缓存
     */
    public void removeOrgCache();

    /**
     * 删除系统参数数据
     *
     * @param schoolId
     * @return
     */
    String removeSysSetting(Long schoolId);

    /**
     * 获取学校缓存
     */
    Map<String, SimpleObject> getSysSetting(Long schoolId);

    String set(String key, String value);

    String get(String key);

    void remove(String key);
}
