package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.bean.dto.CloudMarkingScoreDto;
import com.qmth.eds.bean.dto.ExamSyncStudentDto;
import com.qmth.eds.bean.dto.ScoreDownloadDto;
import com.qmth.eds.bean.result.CourseResult;
import com.qmth.eds.bean.result.StudentMarkResult;
import com.qmth.eds.common.entity.CloudMarkingScore;
import com.qmth.eds.mapper.CloudMarkingScoreMapper;
import com.qmth.eds.service.CloudMarkingScoreService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class CloudMarkingScoreServiceImpl extends ServiceImpl<CloudMarkingScoreMapper, CloudMarkingScore> implements CloudMarkingScoreService {

    @Resource
    CloudMarkingScoreMapper cloudMarkingScoreMapper;

    @Override
    public void deleteByKeys(Long schoolId, Long semesterId, Long examTypeId, Integer examId) {
        UpdateWrapper<CloudMarkingScore> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().eq(CloudMarkingScore::getSchoolId, schoolId)
                .eq(CloudMarkingScore::getSemesterId, semesterId)
                .eq(CloudMarkingScore::getExamTypeId, examTypeId)
                .eq(CloudMarkingScore::getExamId, examId);
        this.remove(updateWrapper);
    }

    @Override
    public IPage<CloudMarkingScoreDto> pageData(Long collegeId, Long semesterId, Long examTypeId, String examId, String syncCourseCode, String cloudMarkingCourseCode,  String studentCode, Boolean related, Integer pageNumber, Integer pageSize) {
        Page<CloudMarkingScoreDto> page = new Page<>(pageNumber, pageSize);
        return this.baseMapper.pageData(page, collegeId, semesterId, examTypeId, examId, syncCourseCode, cloudMarkingCourseCode,  studentCode, related);
    }

    @Override
    public List<ExamSyncStudentDto> listExamSyncStudentDtos(Long schoolId, Long semesterId, Long examTypeId, String courseCode, String openCollege) {
        return this.baseMapper.listExamSyncStudentDtos(schoolId, semesterId, examTypeId, courseCode, openCollege);
    }

    /**
     * 查询学生成绩
     *
     * @param semesterId
     * @param examTypeId
     * @param openCollege
     * @param courseCode
     * @return
     */
    @Override
    public List<StudentMarkResult> queryStudentMark(Long semesterId, Long examTypeId, Long collegeId, String openCollege, String courseCode) {
        return cloudMarkingScoreMapper.queryStudentMark(collegeId, semesterId, examTypeId, openCollege, courseCode);
    }

    @Override
    public List<ScoreDownloadDto> listScoreBySemesterIdAndExamTypeIdAndCollegeIdAndCourseCodeAndOpenCollege(Long semesterId, Long examTypeId, Long collegeId, String courseCode, String openCollege) {
        return this.baseMapper.listScoreBySemesterIdAndExamTypeIdAndCollegeIdAndCourseCodeAndOpenCollege(semesterId, examTypeId, collegeId, courseCode, openCollege);
    }

    @Override
    public List<CloudMarkingScore> listByCollegeIdAndSemesterIdAndExamTypeIdAndSubjectCodeAndOpenCollege(Long collegeId, Long semesterId, Long examTypeId, String courseCode, String openCollege) {
        QueryWrapper<CloudMarkingScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CloudMarkingScore::getSchoolId, collegeId)
                .eq(CloudMarkingScore::getSemesterId, semesterId)
                .eq(CloudMarkingScore::getExamTypeId, examTypeId)
                .eq(CloudMarkingScore::getSubjectCode, courseCode)
                .eq(CloudMarkingScore::getOpenCollege, openCollege);
        return this.list(queryWrapper);
    }

    @Override
    public List<CourseResult> conditionCourseQuery(Long semesterId, Long examTypeId, Long collegeId) {
        return this.baseMapper.conditionCourseQuery(semesterId, examTypeId, collegeId);
    }
}
