package com.qmth.eds.service.impl;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.core.solar.model.AppInfo;
import com.qmth.boot.core.solar.model.OrgInfo;
import com.qmth.boot.core.solar.service.SolarService;
import com.qmth.eds.bean.auth.AuthBean;
import com.qmth.eds.bean.dto.AuthOrgInfoDto;
import com.qmth.eds.bean.dto.syssetting.SettingObject;
import com.qmth.eds.bean.dto.syssetting.SimpleObject;
import com.qmth.eds.bean.result.MenuResult;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.*;
import com.qmth.eds.common.enums.PrivilegePropertyEnum;
import com.qmth.eds.service.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.CachePut;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 缓存操作serviceImpl 实现类
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2021/3/25
 */
@Service
public class CommonCacheServiceImpl implements CommonCacheService {
    private final static Logger log = LoggerFactory.getLogger(CommonCacheServiceImpl.class);

    @Resource
    SysUserService sysUserService;

    @Resource
    BasicSchoolService basicSchoolService;

    @Resource
    CommonService commonService;

    @Resource
    SysOrgService sysOrgService;

    @Resource
    SysRoleService sysRoleService;

    @Resource
    SysSettingService sysSettingService;

    @Resource
    SolarService solarService;

    /**
     * 添加角色缓存
     *
     * @param roleId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.ROLE_CACHE + "', #roleId}")
    public SysRole roleCache(Long roleId) {
        return sysRoleService.getById(roleId);
    }

    /**
     * 修改角色缓存
     *
     * @param roleId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.ROLE_CACHE + "', #roleId}")
    public SysRole updateRoleCache(Long roleId) {
        return sysRoleService.getById(roleId);
    }

    /**
     * 删除角色缓存
     *
     * @param roleId
     * @return
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.ROLE_CACHE + "', #roleId}")
    public void removeRoleCache(Long roleId) {

    }

    /**
     * 添加用户菜单缓存
     *
     * @param userId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_MENU_CACHE + "', #userId}")
    public MenuResult userMenuCache(Long userId) {
        return commonService.getUserMenu(userId);
    }

    /**
     * 修改用户菜单缓存
     *
     * @param userId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_MENU_CACHE + "', #userId}")
    public MenuResult updateUserMenuCache(Long userId) {
        return commonService.getUserMenu(userId);
    }

    /**
     * 删除用户菜单缓存
     *
     * @param userId
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_MENU_CACHE + "', #userId}")
    public void removeUserMenuCache(Long userId) {

    }

    /**
     * 添加用户缓存
     *
     * @param userId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_ACCOUNT_CACHE + "', #userId}")
    public SysUser userCache(Long userId) {
        return sysUserService.getById(userId);
    }

    /**
     * 修改用户缓存
     *
     * @param userId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_ACCOUNT_CACHE + "', #userId}")
    public SysUser updateUserCache(Long userId) {
        return sysUserService.getById(userId);
    }

    /**
     * 删除用户缓存
     *
     * @param userId
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_ACCOUNT_CACHE + "', #userId}")
    public void removeUserCache(Long userId) {

    }

    /**
     * 添加用户鉴权缓存
     *
     * @param userId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_OAUTH_CACHE + "', #userId}")
    public AuthBean userAuthCache(Long userId) {
        return commonService.getUserAuth(userId);
    }

    /**
     * 修改用户鉴权缓存
     *
     * @param userId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_OAUTH_CACHE + "', #userId}")
    public AuthBean updateUserAuthCache(Long userId) {
        return commonService.getUserAuth(userId);
    }

    /**
     * 删除用户鉴权缓存
     *
     * @param userId
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_OAUTH_CACHE + "', #userId}")
    public void removeUserAuthCache(Long userId) {

    }

    /**
     * 添加学校缓存
     *
     * @param schoolId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SCHOOL_CACHE + "', #schoolId}")
    public BasicSchool schoolCache(Long schoolId) {
        return basicSchoolService.getById(schoolId);
    }

    /**
     * 添加学校缓存
     *
     * @param code
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SCHOOL_CODE_CACHE + "', #code}")
    public BasicSchool schoolCache(String code) {
        QueryWrapper<BasicSchool> basicSchoolQueryWrapper = new QueryWrapper<>();
        basicSchoolQueryWrapper.lambda().eq(BasicSchool::getCode, code);
        return basicSchoolService.getOne(basicSchoolQueryWrapper);
    }

    /**
     * 添加机构缓存
     *
     * @param orgId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.ORG_CACHE + "', #orgId}")
    public SysOrg orgCache(Long orgId) {
        return sysOrgService.getById(orgId);
    }

    /**
     * 获取鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.PRIVILEGE_URL_CACHE + "', #privilegePropertyEnum}")
    public Set<String> privilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum) {
        return commonService.getPrivilegeUrl(privilegePropertyEnum);
    }

    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.PRIVILEGE_URL_CACHE + "', #privilegePropertyEnum + '-' + #schoolId}")
    public Set<String> privilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum, Long schoolId) {
        return commonService.getPrivilegeUrl(privilegePropertyEnum, schoolId);
    }

    /**
     * 修改鉴权url
     *
     * @param privilegePropertyEnum
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.PRIVILEGE_URL_CACHE + "', #privilegePropertyEnum}")
    public Set<String> updatePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum) {
        return commonService.getPrivilegeUrl(privilegePropertyEnum);
    }

    /**
     * 删除鉴权url
     *
     * @param privilegePropertyEnum
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.PRIVILEGE_URL_CACHE + "', #privilegePropertyEnum}")
    public void removePrivilegeUrlCache(PrivilegePropertyEnum privilegePropertyEnum) {

    }

    /**
     * 获取角色权限缓存
     *
     * @param roleId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.ROLE_PRIVILEGE_CACHE + "', #roleId}")
    public List<SysRolePrivilege> rolePrivilegeCache(Long roleId) {
        return commonService.getRolePrivilege(roleId);
    }

    /**
     * 修改角色权限缓存
     *
     * @param roleId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.ROLE_PRIVILEGE_CACHE + "', #roleId}")
    public List<SysRolePrivilege> updateRolePrivilegeCache(Long roleId) {
        return commonService.getRolePrivilege(roleId);
    }

    /**
     * 删除角色权限缓存
     *
     * @param roleId
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.ROLE_PRIVILEGE_CACHE + "', #roleId}")
    public void removeRolePrivilegeCache(Long roleId) {

    }

    /**
     * 添加用户角色权限缓存
     *
     * @param userId
     * @return
     */
    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_ROLE_PRIVILEGE_CACHE + "', #userId}")
    public List<SysUserRole> userRolePrivilegeCache(Long userId) {
        return commonService.getUserRolePrivilege(userId);
    }

    /**
     * 修改用户角色权限缓存
     *
     * @param userId
     * @return
     */
    @Override
    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_ROLE_PRIVILEGE_CACHE + "', #userId}")
    public List<SysUserRole> updateUserRolePrivilegeCache(Long userId) {
        return commonService.getUserRolePrivilege(userId);
    }

    /**
     * 删除用户角色权限缓存
     *
     * @param userId
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.USER_ROLE_PRIVILEGE_CACHE + "', #userId}")
    public void removeUserRolePrivilegeCache(Long userId) {

    }

    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.AUTH_INFO_CACHE + "', #code}", unless = "#result == null")
    public AuthOrgInfoDto authInfoCache(String code) {
        AppInfo appInfo = solarService.getAppInfo();
        AuthOrgInfoDto authOrgInfoDto = null;
        if (Objects.nonNull(appInfo)) {
            List<OrgInfo> orgInfoList = solarService.getOrgList().stream().filter(s -> Objects.equals(s.getCode(), code)).collect(Collectors.toList());
            if (Objects.nonNull(orgInfoList) && orgInfoList.size() > 0) {
                authOrgInfoDto = new AuthOrgInfoDto(orgInfoList.get(0), appInfo.getControl());
            }
        }
        return authOrgInfoDto;
    }

    @Override
    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.AUTH_INFO_CACHE + "', #code}", condition = "#result != null")
    public AuthOrgInfoDto updateAuthInfoCache(String code) {
        AppInfo appInfo = solarService.getAppInfo();
        AuthOrgInfoDto authOrgInfoDto = null;
        if (Objects.nonNull(appInfo)) {
            List<OrgInfo> orgInfoList = solarService.getOrgList().stream().filter(s -> Objects.equals(s.getCode(), code)).collect(Collectors.toList());
            if (Objects.nonNull(orgInfoList) && orgInfoList.size() > 0) {
                authOrgInfoDto = new AuthOrgInfoDto(orgInfoList.get(0), appInfo.getControl());
            }
        }
        return authOrgInfoDto;
    }

    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.AUTH_INFO_CACHE + "', #code}")
    public void removeAuthInfoCache(String code) {
    }

    /**
     * 删除学校缓存
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SCHOOL_CACHE + "'}", allEntries = true)
    public void removeSchoolIdCache() {
    }

    /**
     * 删除学校缓存
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SCHOOL_CODE_CACHE + "'}", allEntries = true)
    public void removeSchoolCodeCache() {
    }

    /**
     * 删除机构缓存
     */
    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.ORG_CACHE + "'}", allEntries = true)
    public void removeOrgCache() {
    }

    @Override
    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SYS_SETTING_CACHE + "', #schoolId}")
    public String removeSysSetting(Long schoolId) {
        return null;
    }

    @Override
    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SYS_SETTING_CACHE + "', #schoolId}")
    public Map<String, SimpleObject> getSysSetting(Long schoolId) {
        Map<String, SimpleObject> map = new HashMap<>();
        SysSetting sysSetting = sysSettingService.getBySchoolId(schoolId);
        if (sysSetting != null) {
            String parameterObject = sysSetting.getParameterObject();
            List<SettingObject> settingObjects = JSON.parseArray(parameterObject, SettingObject.class);
            for (SettingObject settingObject : settingObjects) {
                for (SimpleObject simpleObject : settingObject.getData()) {
                    map.put(simpleObject.getCode(), simpleObject);
                }
            }
        }
        return map;
    }

    @CachePut(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SYS_SETTING_CACHE + "', #key}")
    @Override
    public String set(String key, String value) {
        return value;
    }

    @Cacheable(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SYS_SETTING_CACHE + "', #key}")
    @Override
    public String get(String key) {
        return null;
    }

    @CacheEvict(value = SystemConstant.SYSTEM_CACHE, key = "{'" + SystemConstant.SYS_SETTING_CACHE + "', #key}")
    @Override
    public void remove(String key) {

    }
}
