package com.qmth.eds.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.eds.bean.dto.ExamAssignDto;
import com.qmth.eds.bean.dto.ScoreDownloadDto;
import com.qmth.eds.bean.result.AssignResultPreviewResult;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.*;
import com.qmth.eds.common.enums.ExamAssignStatusEnum;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.FileUtil;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.ExamAssignMapper;
import com.qmth.eds.service.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.FillPatternType;
import org.apache.poi.ss.usermodel.IndexedColors;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 赋分管理 服务实现类
 * </p>
 */
@Service
public class ExamAssignServiceImpl extends ServiceImpl<ExamAssignMapper, ExamAssign> implements ExamAssignService {

    @Resource
    ExamSyncTotalService examSyncTotalService;

    @Resource
    ExamSyncStudentService examSyncStudentService;

    @Resource
    AsyncTaskService asyncTaskService;

    @Resource
    CloudMarkingScoreService cloudMarkingScoreService;

    @Resource
    CloudMarkingScoreForeignService cloudMarkingScoreForeignService;

    @Resource
    ExamAssignMapper examAssignMapper;

    @Override
    public IPage<ExamAssignDto> pageData(Long semesterId, Long examTypeId, Long collegeId, String courseCode, Integer pageNumber, Integer pageSize) {
        Page<ExamAssignDto> page = new Page<>(pageNumber, pageSize);
        IPage<ExamAssignDto> assignListDtoIPage = this.baseMapper.pageData(page, collegeId, semesterId, examTypeId, courseCode);

        for (ExamAssignDto record : assignListDtoIPage.getRecords()) {
            if (StringUtils.isNotBlank(record.getOpenCollege())) {
                record.setOpenCollege(Arrays.asList(record.getOpenCollege().split(",")).stream().filter(m -> StringUtils.isNotBlank(m)).collect(Collectors.joining(",")));
            }

            ExamAssign examAssign = this.baseMapper.getBySchoolIdAndSemesterIdAndExamTypeIdAndCourseCodeAndOpenCollege(collegeId, semesterId, examTypeId, record.getCourseCode(), record.getOpenCollege());
            if (examAssign != null) {
                record.setId(examAssign.getId());
                record.setFormula(examAssign.getFormula());
                record.setFormulaStr(examAssign.getFormula() != null ? examAssign.getFormula().getName() : "/");
                record.setStatus(examAssign.getStatus());
                record.setStatusStr(examAssign.getStatus().getDesc());
                record.setResultStr(ExamAssignStatusEnum.FINISH.equals(examAssign.getStatus()) ? StringUtils.isBlank(examAssign.getErrorMsg()) ? "成功" : "失败" : "/");
                record.setPublish(examAssign.getPublish());
            }
        }

        return assignListDtoIPage;
    }

    @Override
    public String calcData(String data) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        ExamAssign examAssign = JSON.parseObject(data, ExamAssign.class);
        // 保存赋分参数
        if (examAssign.getId() == null) {
            examAssign.setId(SystemConstant.getDbUuid());
            examAssign.setStatus(ExamAssignStatusEnum.INIT);
            examAssign.setCreateId(sysUser.getId());
            examAssign.setCreateTime(System.currentTimeMillis());
        } else {
            examAssign.setUpdateId(sysUser.getId());
            examAssign.setUpdateTime(System.currentTimeMillis());
        }
        this.saveOrUpdate(examAssign);
        examAssign = this.getById(examAssign.getId());

        // 计算赋分
        asyncTaskService.calcAssignScore(examAssign);
        return String.valueOf(examAssign.getId());
    }

    /**
     * 赋分结果预览
     *
     * @param iPage
     * @param semesterId
     * @param examTypeId
     * @param courseCode
     * @param openCollege
     * @param kcCollege
     * @param clazzId
     * @param teacherCode
     * @param minAssignScore
     * @param maxAssignScore
     * @return
     */
    @Override
    public IPage<AssignResultPreviewResult> assignResultPreview(IPage<Map> iPage,
                                                                Long semesterId,
                                                                Long examTypeId,
                                                                Long collegeId,
                                                                String courseCode,
                                                                String openCollege,
                                                                String kcCollege,
                                                                String clazzId,
                                                                String teacherCode,
                                                                String minAssignScore,
                                                                String maxAssignScore) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        return examAssignMapper.assignResultPreview(iPage, schoolId, collegeId, semesterId, examTypeId, courseCode, openCollege, kcCollege, clazzId, teacherCode, minAssignScore, maxAssignScore);
    }

    @Override
    public Map contrast(Long semesterId, Long examTypeId, String courseCode) {
        return null;
    }

    @Override
    public ExamAssignDto toCalc(Long semesterId, Long examTypeId, Long collegeId, String syncCourseCode, String courseCode, String courseName, String openCollege) {
        if (StringUtils.isBlank(openCollege)) {
            throw ExceptionResultEnum.ERROR.exception("未查询到开课学院，无法进行计算，请联系管理员处理");
        }
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());

        ExamAssign examAssign = this.baseMapper.getBySchoolIdAndSemesterIdAndExamTypeIdAndCourseCodeAndOpenCollege(collegeId, semesterId, examTypeId, courseCode, openCollege);
        ExamAssignDto examAssignDto = new ExamAssignDto();
        if (examAssign == null) {
            examAssign = new ExamAssign(collegeId, semesterId, examTypeId, syncCourseCode, courseCode, courseName, openCollege, null, null, ExamAssignStatusEnum.INIT, false);
            this.save(examAssign);
        }
        BeanUtils.copyProperties(examAssign, examAssignDto);

        ExamSyncTotal examSyncTotal = examSyncTotalService.getBySemesterIdAndExamTypeIdAndUseFile(schoolId, semesterId, examTypeId, true);
        if (examSyncTotal == null) {
            throw ExceptionResultEnum.ERROR.exception("未设置使用数据文件，请联系管理员处理");
        }
        int actualCount = 0;
        List<String> kcxyList = new ArrayList<>();
        List<CloudMarkingScore> cloudMarkingScoreList = cloudMarkingScoreService.listByCollegeIdAndSemesterIdAndExamTypeIdAndSubjectCodeAndOpenCollege(collegeId, semesterId, examTypeId, courseCode, openCollege);
        if (!CollectionUtils.isEmpty(cloudMarkingScoreList)) {
            actualCount = cloudMarkingScoreList.size();
            kcxyList = cloudMarkingScoreList.stream().filter(m -> StringUtils.isNotBlank(m.getCollege())).map(CloudMarkingScore::getCollege).distinct().collect(Collectors.toList());
        }
        examAssignDto.setActualCount(actualCount);
        examAssignDto.setInspectCollege(kcxyList);
        return examAssignDto;
    }

    @Transactional
    @Override
    public boolean publish(Long id, Long semesterId, Long examTypeId, Long collegeId, String syncCourseCode, String courseCode, String courseName, String openCollege) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();

        ExamAssign examAssign;
        if (id == null) {
            examAssign = new ExamAssign(collegeId, semesterId, examTypeId, syncCourseCode, courseCode, courseName, openCollege, null, null, ExamAssignStatusEnum.INIT, false);
            examAssign.insertInfo(sysUser.getId());
            this.save(examAssign);
        } else {
            examAssign = this.getById(id);
            if (examAssign.getFormula() != null && !ExamAssignStatusEnum.FINISH.equals(examAssign.getStatus())) {
                throw ExceptionResultEnum.ERROR.exception("赋分计算未完成，暂无法发布成绩");
            }
        }

        // 查询对外数据
        ExamSyncTotal examSyncTotal = examSyncTotalService.getBySemesterIdAndExamTypeIdAndUseFile(schoolId, semesterId, examTypeId, true);
        if (examSyncTotal == null) {
            throw ExceptionResultEnum.ERROR.exception("未设置使用数据文件，请联系管理员处理");
        }

        QueryWrapper<CloudMarkingScore> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(CloudMarkingScore::getSchoolId, collegeId)
                .eq(CloudMarkingScore::getSemesterId, semesterId)
                .eq(CloudMarkingScore::getExamTypeId, examTypeId)
                .eq(CloudMarkingScore::getSubjectCode, courseCode);
        List<CloudMarkingScore> cloudMarkingScoreList = cloudMarkingScoreService.list(queryWrapper);

        List<CloudMarkingScoreForeign> cloudMarkingScoreForeigns = new ArrayList<>();
        List<ExamSyncStudent> examSyncStudents = examSyncStudentService.listByExamSyncTotalId(examSyncTotal);
        for (CloudMarkingScore cloudMarkingScore : cloudMarkingScoreList) {
            if (cloudMarkingScore.getExamSyncStudentId() != null) {
                ExamSyncStudent examSyncStudent = examSyncStudents.stream().filter(m -> cloudMarkingScore.getExamSyncStudentId().equals(m.getId())).findFirst().get();
                cloudMarkingScoreForeigns.add(new CloudMarkingScoreForeign(collegeId, semesterId, examTypeId, cloudMarkingScore.getExamId(), examSyncStudent.getXnm(), examSyncStudent.getXqm(), examSyncStudent.getJxbId(), examSyncStudent.getKch(), examSyncStudent.getKcmc(), examSyncStudent.getXh(), cloudMarkingScore.getAssignScore()));
            } else {
                Optional<ExamSyncStudent> optional = examSyncStudents.stream()
                        .filter(s -> s.getKch().equals(cloudMarkingScore.getSyncCourseCode()) && s.getXh().equals(cloudMarkingScore.getStudentCode())).findFirst();
                if (!optional.isPresent()) {
                    cloudMarkingScoreForeigns.add(new CloudMarkingScoreForeign(collegeId, semesterId, examTypeId, cloudMarkingScore.getExamId(), null, null, null, cloudMarkingScore.getSyncCourseCode(), cloudMarkingScore.getSubjectName(), cloudMarkingScore.getStudentCode(), cloudMarkingScore.getAssignScore()));
                } else {
                    ExamSyncStudent examSyncStudent = optional.get();
                    // 匹配上，保存考务数据表ID
                    cloudMarkingScoreForeigns.add(new CloudMarkingScoreForeign(collegeId, semesterId, examTypeId, cloudMarkingScore.getExamId(), examSyncStudent.getXnm(), examSyncStudent.getXqm(), examSyncStudent.getJxbId(), examSyncStudent.getKch(), examSyncStudent.getKcmc(), examSyncStudent.getXh(), cloudMarkingScore.getAssignScore()));
                }
            }
        }
        // 重新发布时，删除已发布的成绩
        cloudMarkingScoreForeignService.deleteByCourseCode(collegeId, semesterId, examTypeId, syncCourseCode, courseCode);

        List<CloudMarkingScoreForeign> foreignData = new ArrayList<>();
        //1000条提交一次
        for (CloudMarkingScoreForeign scoreForeign : cloudMarkingScoreForeigns) {
            if (foreignData.size() == 1000) {
                cloudMarkingScoreForeignService.saveBatch(foreignData);
                foreignData.clear();
            }
            foreignData.add(scoreForeign);
        }
        //将剩下的数据也导入
        if (!foreignData.isEmpty()) {
            cloudMarkingScoreForeignService.saveBatch(foreignData);
        }

        // 更新发布状态为已发布
        UpdateWrapper<ExamAssign> examAssignUpdateWrapper = new UpdateWrapper<>();
        examAssignUpdateWrapper.lambda().set(ExamAssign::getPublish, true)
                .eq(ExamAssign::getId, examAssign.getId());
        return this.update(examAssignUpdateWrapper);
    }

    @Override
    public void download(Long id, Long semesterId, Long examTypeId, Long collegeId, String courseCode, String openCollege, HttpServletResponse response) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        try {
            String fileNamePrefix = "批量导出" + System.currentTimeMillis();
            if (id != null) {
                ExamAssign examAssign = this.getById(id);
                if (examAssign != null) {
                    fileNamePrefix = examAssign.getCourseName() + "(" + examAssign.getCourseCode() + ")";
                }
            }
            List<ScoreDownloadDto> scoreDownloadDtoList = cloudMarkingScoreService.listScoreBySemesterIdAndExamTypeIdAndCollegeIdAndCourseCodeAndOpenCollege(semesterId, examTypeId, collegeId, courseCode, openCollege);
            response.setHeader("Content-Disposition",
                    "inline;filename=" + URLEncoder.encode(fileNamePrefix, SystemConstant.CHARSET_NAME) + ".xlsx");
            response.setContentType("application/vnd.ms-excel");
            ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);

            String sheetName = "赋分成绩数据";
            // 设置样式
            CellStyle cellStyle1 = writer.createCellStyle();
            org.apache.poi.ss.usermodel.Font font1 = writer.createFont();
            font1.setBold(true);
            font1.setFontHeightInPoints((short) 18);
            cellStyle1.setFont(font1);
            cellStyle1.setFillForegroundColor(IndexedColors.GREY_25_PERCENT.getIndex());
//            cellStyle1.setFillPattern(FillPatternType.SOLID_FOREGROUND);
            writer.setCellStyle(sheetName, cellStyle1, 0, new int[]{0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12});

            writer.writeObjects(sheetName, null, ScoreDownloadDto.class, scoreDownloadDtoList.listIterator());
            writer.output(response.getOutputStream());
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
