package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.ExamScheduleTask;
import com.qmth.eds.common.entity.ExamSemester;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.ExamSemesterMapper;
import com.qmth.eds.service.ExamScheduleTaskService;
import com.qmth.eds.service.ExamSemesterService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class ExamSemesterServiceImpl extends ServiceImpl<ExamSemesterMapper, ExamSemester> implements ExamSemesterService {

    @Resource
    ExamScheduleTaskService examScheduleTaskService;

    @Override
    public List<ExamSemester> listBySchoolId() {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamSemester> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamSemester::getSchoolId, schoolId)
                .eq(ExamSemester::getEnable, true)
                .orderByDesc(ExamSemester::getCreateTime);
        return this.baseMapper.selectList(queryWrapper);
    }

    @Override
    public boolean saveData(ExamSemester examSemester) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        if (examSemester.getId() == null) {
            examSemester.setId(SystemConstant.getDbUuid());
            examSemester.setSchoolId(schoolId);
            return this.save(examSemester);
        } else {
            QueryWrapper<ExamSemester> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamSemester::getSchoolId, schoolId)
                    .eq(ExamSemester::getName, examSemester.getName())
                    .ne(ExamSemester::getId, examSemester.getId());
            List<ExamSemester> examSemesters = this.list(queryWrapper);
            if (!examSemesters.isEmpty()) {
                throw ExceptionResultEnum.ERROR.exception("名称已存在");
            }
            return this.updateById(examSemester);
        }
    }

    @Override
    public boolean delete(Long id) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamScheduleTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamScheduleTask::getSchoolId, schoolId)
                .eq(ExamScheduleTask::getSemesterId, id);
        List<ExamScheduleTask> examScheduleTasks = examScheduleTaskService.list(queryWrapper);
        if (!examScheduleTasks.isEmpty()) {
            throw ExceptionResultEnum.ERROR.exception("学期已在定时任务中使用，不能删除");
        }
        return this.removeById(id);
    }

    @Override
    public IPage<ExamSemester> pageData(Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        Page<ExamSemester> page = new Page<>(pageNumber, pageSize);
        QueryWrapper<ExamSemester> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamSemester::getSchoolId, schoolId);
        return this.page(page, queryWrapper);
    }

    @Override
    public boolean enable(Long id) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        ExamSemester examSemester = this.getById(id);

        if (examSemester.getEnable()) {
            long currentTime = System.currentTimeMillis();
            QueryWrapper<ExamScheduleTask> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(ExamScheduleTask::getSchoolId, schoolId)
                    .eq(ExamScheduleTask::getSemesterId, id)
                    .lt(ExamScheduleTask::getStartTime, currentTime)
                    .gt(ExamScheduleTask::getEndTime, currentTime);
            int count = examScheduleTaskService.count(queryWrapper);
            if (count != 0) {
                throw ExceptionResultEnum.ERROR.exception("该学期下有未结束的定时任务，无法禁用");
            }
        }

        examSemester.setEnable(!examSemester.getEnable());
        return this.updateById(examSemester);
    }
}
