package com.qmth.eds.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.eds.bean.dto.ExamSyncTotalDownloadDto;
import com.qmth.eds.common.contant.SystemConstant;
import com.qmth.eds.common.entity.*;
import com.qmth.eds.common.enums.ExceptionResultEnum;
import com.qmth.eds.common.util.FileUtil;
import com.qmth.eds.common.util.ServletUtil;
import com.qmth.eds.mapper.ExamSyncTotalMapper;
import com.qmth.eds.service.*;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class ExamSyncTotalServiceImpl extends ServiceImpl<ExamSyncTotalMapper, ExamSyncTotal> implements ExamSyncTotalService {

    @Resource
    private SysUserService sysUserService;

    @Resource
    private ExamSemesterService examSemesterService;

    @Resource
    private ExamTypeService examTypeService;

    @Resource
    private ExamDownloadRecordService examDownloadRecordService;

    @Override
    public IPage<ExamSyncTotal> list(Long semesterId, Long examTypeId, Long syncDate, Integer pageNumber, Integer pageSize) {
        Long schoolId = Long.valueOf(ServletUtil.getRequestHeaderSchoolId().toString());
        QueryWrapper<ExamSyncTotal> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamSyncTotal::getSchoolId, schoolId);
        if (semesterId != null) {
            queryWrapper.lambda().eq(ExamSyncTotal::getSemesterId, semesterId);
        } else {
            List<ExamSemester> examSemesters = examSemesterService.listBySchoolId();
            List<Long> semesterIds = new ArrayList<>();
            if (!CollectionUtils.isEmpty(examSemesters)) {
                semesterIds = examSemesters.stream().map(ExamSemester::getId).collect(Collectors.toList());
            }
            queryWrapper.lambda().in(ExamSyncTotal::getSemesterId, semesterIds);
        }
        if (examTypeId != null) {
            queryWrapper.lambda().eq(ExamSyncTotal::getExamTypeId, examTypeId);
        } else {
            List<ExamType> examTypes = examTypeService.listData();
            List<Long> examTypeIds = new ArrayList<>();
            if (!CollectionUtils.isEmpty(examTypes)) {
                examTypeIds = examTypes.stream().map(ExamType::getId).collect(Collectors.toList());
            }
            queryWrapper.lambda().in(ExamSyncTotal::getExamTypeId, examTypeIds);
        }

        if (syncDate != null) {
            queryWrapper.lambda().eq(ExamSyncTotal::getSyncDate, syncDate);
        }
        queryWrapper.lambda().orderByDesc(ExamSyncTotal::getCreateTime);
        return this.baseMapper.selectPage(new Page<>(pageNumber, pageSize), queryWrapper);
    }

    @Override
    public void download(Long id, Boolean writeLog, HttpServletResponse response) {
        ExamSyncTotal examSyncTotal = this.getById(id);
        String fileName = examSyncTotal.getFileName();
        String filePath = examSyncTotal.getFilePath();
        File file = new File(filePath);
        if (!file.exists()) {
            throw ExceptionResultEnum.ERROR.exception("文件不存在");
        }

        if (writeLog) {
            // 记录日志
            Long userId = ServletUtil.getRequestUserId();
            SysUser sysUser = sysUserService.getById(userId);

            ExamDownloadRecord examDownloadRecord = new ExamDownloadRecord();
            examDownloadRecord.setId(SystemConstant.getDbUuid());
            examDownloadRecord.setOperateTime(System.currentTimeMillis());
            examDownloadRecord.setUserId(userId);
            examDownloadRecord.setUserName(sysUser.getRealName());
            examDownloadRecord.setSyncTotalId(id);
            examDownloadRecord.setUsed(false);
            examDownloadRecord.setCreateId(userId);
            examDownloadRecord.setCreateTime(System.currentTimeMillis());
            examDownloadRecordService.save(examDownloadRecord);
        }

        // 下载
        FileUtil.outputFile(response, file, fileName);
    }

    @Override
    public void updateDownloadStatusBySchoolIdAndSemesterIdAndExamTypeId(Long schoolId, Long semesterId, Long examTypeId) {
        UpdateWrapper<ExamSyncTotal> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamSyncTotal::getDownloadStatus, false)
                .eq(ExamSyncTotal::getSchoolId, schoolId)
                .eq(ExamSyncTotal::getSemesterId, semesterId)
                .eq(ExamSyncTotal::getExamTypeId, examTypeId);
        this.update(updateWrapper);
    }

    @Override
    public List<ExamSyncTotalDownloadDto> listDownloadRecordInfos(Long schoolId, Long semesterId, Long examTypeId) {
        return this.baseMapper.listDownloadRecordInfos(schoolId, semesterId, examTypeId);
    }

    @Override
    public ExamSyncTotal getBySemesterIdAndExamTypeIdAndDownloadStatus(Long schoolId, Long semesterId, Long examTypeId, boolean status) {
        QueryWrapper<ExamSyncTotal> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamSyncTotal::getSchoolId, schoolId)
                .eq(ExamSyncTotal::getSemesterId, semesterId)
                .eq(ExamSyncTotal::getExamTypeId, examTypeId)
                .eq(ExamSyncTotal::getDownloadStatus, status);
        return this.getOne(queryWrapper);
    }

    @Override
    public ExamSyncTotal getBySemesterIdAndExamTypeIdAndUseFile(Long schoolId, Long semesterId, Long examTypeId, boolean status) {
        QueryWrapper<ExamSyncTotal> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamSyncTotal::getSchoolId, schoolId)
                .eq(ExamSyncTotal::getSemesterId, semesterId)
                .eq(ExamSyncTotal::getExamTypeId, examTypeId)
                .eq(ExamSyncTotal::getUseFile, status);
        return this.getOne(queryWrapper);
    }

    @Transactional
    @Override
    public boolean used(Long id) {
        ExamSyncTotal examSyncTotal = this.getById(id);
        // 将其它已设置为true的数据改为false
        UpdateWrapper<ExamSyncTotal> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamSyncTotal::getUseFile, false)
                .eq(ExamSyncTotal::getSchoolId, examSyncTotal.getSchoolId())
                .eq(ExamSyncTotal::getSemesterId, examSyncTotal.getSemesterId())
                .eq(ExamSyncTotal::getExamTypeId, examSyncTotal.getExamTypeId())
                .eq(ExamSyncTotal::getUseFile, true);
        this.update(updateWrapper);

        UpdateWrapper<ExamSyncTotal> updateWrapper1 = new UpdateWrapper<>();
        updateWrapper1.lambda().set(ExamSyncTotal::getUseFile, true)
                .eq(ExamSyncTotal::getId, id);
        return this.update(updateWrapper1);
    }
}
